@@ -6,7 +6,7 @@
 #	  /_/  |_/_/\__,_/\___/\____/_/   \__,_/\___/_/     
 #
 #
-#		ALACORDER beta 7.4.4
+#		ALACORDER beta 7.4.8
 #		by Sam Robson
 #
 #
@@ -16,6 +16,8 @@ import sys
 import glob
 import re
 import math
+import numexpr
+import bottleneck
 import numpy as np
 import pandas as pd
 import xlrd
@@ -52,6 +54,8 @@ def config(in_path: str, out_path: str, flags="", print_log=True, warn=False, sa
 	if out_ext == "txt":
 		print("WARNING: Text files cannot be reimported to alacorder!")
 
+	fromArchive = False
+
 	# Set read, write modes, contents
 	if in_ext == "directory" and bool(out_ext == "pkl" or out_ext == "xz" or out_ext == "txt"): 
 		make = "archive"
@@ -67,11 +71,13 @@ def config(in_path: str, out_path: str, flags="", print_log=True, warn=False, sa
 	elif in_ext == "pkl":
 		make = "table"
 		origin = "archive"
-		contents = pd.read_pickle(in_path)['Path']
+		contents = pd.read_pickle(in_path)['AllPagesText']
+		fromArchive = True
 	elif in_ext == "xz":
 		make = "table"
 		origin = "archive"
-		contents = pd.read_pickle(in_path,compression="xz")['Path']
+		contents = pd.read_pickle(in_path,compression="xz")['AllPagesText']
+		fromArchive = True
 	elif in_ext == "csv":
 		make = "table"
 		origin = "archive"
@@ -105,6 +111,7 @@ def config(in_path: str, out_path: str, flags="", print_log=True, warn=False, sa
 		'out_path': out_path,
 		'in_ext': in_ext,
 		'out_ext': out_ext,
+		'archive': fromArchive,
 		'origin': origin,
 		'make': make,
 		'contents': contents,
@@ -146,7 +153,6 @@ def writeArchive(conf):
 	paths = pd.Series(contents)
 	allpagestext = paths.map(lambda x: getPDFText(x))
 	timestamp = time.time()
-	console_log(conf, on_batch,exptime,"Reading full text of cases...")
 
 	c = pd.DataFrame({
 		'Path': paths,
@@ -191,6 +197,7 @@ def writeTables(conf):
 	contents = conf['contents']
 	batches = conf['batches']
 	save_archive = conf['save_archive']
+	from_archive = conf['archive']
 	if warn == False:
 		warnings.filterwarnings("ignore")
 	start_time = time.time()
@@ -201,122 +208,134 @@ def writeTables(conf):
 	charges = pd.DataFrame({'CaseNumber': '', 'Num': '', 'Code': '', 'Felony': '', 'Conviction': '', 'CERV': '', 'Pardon': '', 'Permanent': '', 'Disposition': '', 'CourtActionDate': '', 'CourtAction': '', 'Cite': '', 'TypeDescription': '', 'Category': '', 'Description': ''},index=[0]) 
 	arch = pd.DataFrame({'Path':'','AllPagesText':'','Timestamp':''},index=[0])
 
-	for i, c in enumerate(batches):
-		b = pd.DataFrame()
-		b['AllPagesText'] = pd.Series(c).map(lambda x: getPDFText(x))
-		exptime = time.time()
-		if save_archive == True:
-			timestamp = exptime
-			ar = pd.DataFrame({
-				'Path': c,
-				'AllPagesText': b['AllPagesText'],
-				'Timestamp': timestamp
-				},index=range(0,batchsize))
-			arch = pd.concat([arch, ar],ignore_index=True)
-			arch.fillna('',inplace=True)
-			arch.dropna(inplace=True)
-			arch.to_pickle(path_out+".pkl.xz",compression="xz")
+	b = pd.DataFrame()
+	exptime = time.time()
 
-		b['CaseInfoOutputs'] = b['AllPagesText'].map(lambda x: getCaseInfo(x))
-		b['CaseNumber'] = b['CaseInfoOutputs'].map(lambda x: x[0])
-		b['Name'] = b['CaseInfoOutputs'].map(lambda x: x[1])
-		b['Alias'] = b['CaseInfoOutputs'].map(lambda x: x[2])
-		b['DOB'] = b['CaseInfoOutputs'].map(lambda x: x[3])
-		b['Race'] = b['CaseInfoOutputs'].map(lambda x: x[4])
-		b['Sex'] = b['CaseInfoOutputs'].map(lambda x: x[5])
-		b['Address'] = b['CaseInfoOutputs'].map(lambda x: x[6])
-		b['Phone'] = b['CaseInfoOutputs'].map(lambda x: x[7])
-		b['ChargesOutputs'] = b.index.map(lambda x: getCharges(b.loc[x].AllPagesText, b.loc[x].CaseNumber))
-		b['Convictions'] = b['ChargesOutputs'].map(lambda x: x[0])
-		b['DispositionCharges'] = b['ChargesOutputs'].map(lambda x: x[1])
-		b['FilingCharges'] = b['ChargesOutputs'].map(lambda x: x[2])
-		b['CERVConvictions'] = b['ChargesOutputs'].map(lambda x: x[3])
-		b['PardonConvictions'] = b['ChargesOutputs'].map(lambda x: x[4])
-		b['PermanentConvictions'] = b['ChargesOutputs'].map(lambda x: x[5])
-		b['ConvictionCount'] = b['ChargesOutputs'].map(lambda x: x[6])
-		b['ChargeCount'] = b['ChargesOutputs'].map(lambda x: x[7])
-		b['CERVChargeCount'] = b['ChargesOutputs'].map(lambda x: x[8])
-		b['PardonChargeCount'] = b['ChargesOutputs'].map(lambda x: x[9])
-		b['PermanentChargeCount'] = b['ChargesOutputs'].map(lambda x: x[10])
-		b['CERVConvictionCount'] = b['ChargesOutputs'].map(lambda x: x[11])
-		b['PardonConvictionCount'] = b['ChargesOutputs'].map(lambda x: x[12])
-		b['PermanentConvictionCount'] = b['ChargesOutputs'].map(lambda x: x[13])
-		b['ChargeCodes'] = b['ChargesOutputs'].map(lambda x: x[14])
-		b['ConvictionCodes'] = b['ChargesOutputs'].map(lambda x: x[15])
-		b['FeeOutputs'] = b.index.map(lambda x: getFeeSheet(b.loc[x].AllPagesText, b.loc[x].CaseNumber))
-		b['TotalAmtDue'] = b['FeeOutputs'].map(lambda x: x[0])
-		b['TotalBalance'] = b['FeeOutputs'].map(lambda x: x[1])
-		b['TotalD999'] = b['FeeOutputs'].map(lambda x: x[2])
-		b['FeeCodesOwed'] = b['FeeOutputs'].map(lambda x: x[3])
-		b['FeeCodes'] = b['FeeOutputs'].map(lambda x: x[4])
-		b['FeeSheet'] = b['FeeOutputs'].map(lambda x: x[5])
+	if from_archive == True:
+		b['AllPagesText'] = contents.str.slice(0,100)
+	else:
+		b['AllPagesText'] = pd.Series(contents).map(lambda x: getPDFText(x))
 
-		fees['AmtDue'] = fees['AmtDue'].map(lambda x: pd.to_numeric(x,'ignore'))
-		fees['AmtPaid'] = fees['AmtPaid'].map(lambda x: pd.to_numeric(x,'ignore'))
-		fees['Balance'] = fees['Balance'].map(lambda x: pd.to_numeric(x,'ignore'))
-		fees['AmtHold'] = fees['AmtHold'].map(lambda x: pd.to_numeric(x,'ignore'))
-		charges['Num'] = charges['Num'].map(lambda x: pd.to_numeric(x,'ignore'))
+	b['CaseInfoOutputs'] = b['AllPagesText'].map(lambda x: getCaseInfo(x))
+	b['CaseNumber'] = b['CaseInfoOutputs'].map(lambda x: x[0])
+	b['Name'] = b['CaseInfoOutputs'].map(lambda x: x[1])
+	b['Alias'] = b['CaseInfoOutputs'].map(lambda x: x[2])
+	b['DOB'] = b['CaseInfoOutputs'].map(lambda x: x[3])
+	b['Race'] = b['CaseInfoOutputs'].map(lambda x: x[4])
+	b['Sex'] = b['CaseInfoOutputs'].map(lambda x: x[5])
+	b['Address'] = b['CaseInfoOutputs'].map(lambda x: x[6])
+	b['Phone'] = b['CaseInfoOutputs'].map(lambda x: x[7])
+	b['ChargesOutputs'] = b.index.map(lambda x: getCharges(b.loc[x].AllPagesText, b.loc[x].CaseNumber))
+	b['Convictions'] = b['ChargesOutputs'].map(lambda x: x[0])
+	b['DispositionCharges'] = b['ChargesOutputs'].map(lambda x: x[1])
+	b['FilingCharges'] = b['ChargesOutputs'].map(lambda x: x[2])
+	b['CERVConvictions'] = b['ChargesOutputs'].map(lambda x: x[3])
+	b['PardonConvictions'] = b['ChargesOutputs'].map(lambda x: x[4])
+	b['PermanentConvictions'] = b['ChargesOutputs'].map(lambda x: x[5])
+	b['ConvictionCount'] = b['ChargesOutputs'].map(lambda x: x[6])
+	b['ChargeCount'] = b['ChargesOutputs'].map(lambda x: x[7])
+	b['CERVChargeCount'] = b['ChargesOutputs'].map(lambda x: x[8])
+	b['PardonChargeCount'] = b['ChargesOutputs'].map(lambda x: x[9])
+	b['PermanentChargeCount'] = b['ChargesOutputs'].map(lambda x: x[10])
+	b['CERVConvictionCount'] = b['ChargesOutputs'].map(lambda x: x[11])
+	b['PardonConvictionCount'] = b['ChargesOutputs'].map(lambda x: x[12])
+	b['PermanentConvictionCount'] = b['ChargesOutputs'].map(lambda x: x[13])
+	b['ChargeCodes'] = b['ChargesOutputs'].map(lambda x: x[14])
+	b['ConvictionCodes'] = b['ChargesOutputs'].map(lambda x: x[15])
+	b['FeeOutputs'] = b.index.map(lambda x: getFeeSheet(b.loc[x].AllPagesText, b.loc[x].CaseNumber))
+	b['TotalAmtDue'] = b['FeeOutputs'].map(lambda x: x[0])
+	b['TotalBalance'] = b['FeeOutputs'].map(lambda x: x[1])
+	b['TotalD999'] = b['FeeOutputs'].map(lambda x: x[2])
+	b['FeeCodesOwed'] = b['FeeOutputs'].map(lambda x: x[3])
+	b['FeeCodes'] = b['FeeOutputs'].map(lambda x: x[4])
+	b['FeeSheet'] = b['FeeOutputs'].map(lambda x: x[5])
 
-		feesheet = b['FeeOutputs'].map(lambda x: x[6]) 
+	fees['AmtDue'] = fees['AmtDue'].map(lambda x: pd.to_numeric(x,'ignore'))
+	fees['AmtPaid'] = fees['AmtPaid'].map(lambda x: pd.to_numeric(x,'ignore'))
+	fees['Balance'] = fees['Balance'].map(lambda x: pd.to_numeric(x,'ignore'))
+	fees['AmtHold'] = fees['AmtHold'].map(lambda x: pd.to_numeric(x,'ignore'))
+	charges['Num'] = charges['Num'].map(lambda x: pd.to_numeric(x,'ignore'))
 
-		feesheet= feesheet.dropna() 
-		fees=fees.dropna()
-		feesheet = feesheet.tolist() # -> [df, df, df]
+	feesheet = b['FeeOutputs'].map(lambda x: x[6]) 
+
+	feesheet= feesheet.dropna() 
+	fees=fees.dropna()
+	feesheet = feesheet.tolist() # -> [df, df, df]
+	try:
 		feesheet = pd.concat(feesheet,axis=0,ignore_index=True) #  -> batch df
+	except ValueError:
+		pass
+	try:
 		fees = fees.append(feesheet, ignore_index=True) # -> all fees df
-
-		if print_log == True:
-			print(fees)
-
-		chargetabs = b['ChargesOutputs'].map(lambda x: x[17])
-		chargetabs = chargetabs.dropna()
-		charges = charges.dropna()
-		chargetabs = chargetabs.tolist()
-		chargetabs = pd.concat(chargetabs,axis=0,ignore_index=True)
-		charges = charges.append(chargetabs,ignore_index=True)
-
-		if print_log == True:
-			print(charges)
-
-		on_batch += 1
-		console_log(conf, on_batch,exptime,'Exporting detailed case information to table...')
-
-		b['ChargesTable'] = b['ChargesOutputs'].map(lambda x: x[-1])
-		b['TotalD999'] = b['TotalD999'].map(lambda x: pd.to_numeric(x,'ignore'))
-		b['Phone'] =  b['Phone'].map(lambda x: pd.to_numeric(x,'ignore'))
-		b['TotalAmtDue'] = b['TotalAmtDue'].map(lambda x: pd.to_numeric(x,'ignore'))
-		b['TotalBalance'] = b['TotalBalance'].map(lambda x: pd.to_numeric(x,'ignore'))
-		b.drop(columns=['AllPagesText','CaseInfoOutputs','ChargesOutputs','FeeOutputs','TotalD999','ChargesTable','FeeSheet'],inplace=True)
-		outputs = pd.concat([outputs, b],ignore_index=True)
+	except ValueError:
+		pass
 		
-		outputs.fillna('',inplace=True)
-		charges.fillna('',inplace=True)
-		fees.fillna('',inplace=True)
+	if print_log == True:
+		print(fees)
 
-		# write 
-		if out_ext == "xls":
-			with pd.ExcelWriter(path_out) as writer:
-				outputs.to_excel(writer, sheet_name="cases-table")
-				fees.to_excel(writer, sheet_name="fees-table")
-				charges.to_excel(writer, sheet_name="charges-table")
-		elif out_ext == "pkl":
-			outputs.to_pickle(path_out+".xz",compression="xz")
-		elif out_ext == "xz":
-			outputs.to_pickle(path_out,compression="xz")
-		elif out_ext == "json":
-			outputs.to_json(path_out)
-		elif out_ext == "csv":
-			outputs.to_csv(path_out,escapechar='\\')
-		elif out_ext == "md":
-			outputs.to_markdown(path_out)
-		elif out_ext == "txt":
-			outputs.to_string(path_out)
-		elif out_ext == "dta":
-			outputs.to_stata(path_out)
-		elif out_ext == "no_export" or print_log == True:
-			print(outputs.to_string())
-		else:
-			raise Exception("Output file extension not supported! Please output to .xls, .pkl, .json, or .csv")
+	chargetabs = b['ChargesOutputs'].map(lambda x: x[17])
+	chargetabs = chargetabs.dropna()
+	charges = charges.dropna()
+	chargetabs = chargetabs.tolist()
+	chargetabs = pd.concat(chargetabs,axis=0,ignore_index=True)
+	charges = charges.append(chargetabs,ignore_index=True)
+
+	if print_log == True:
+		print(charges)
+
+	on_batch += 1
+
+	b['ChargesTable'] = b['ChargesOutputs'].map(lambda x: x[-1])
+	b['TotalD999'] = b['TotalD999'].map(lambda x: pd.to_numeric(x,'ignore'))
+	b['Phone'] =  b['Phone'].map(lambda x: pd.to_numeric(x,'ignore'))
+	b['TotalAmtDue'] = b['TotalAmtDue'].map(lambda x: pd.to_numeric(x,'ignore'))
+	b['TotalBalance'] = b['TotalBalance'].map(lambda x: pd.to_numeric(x,'ignore'))
+
+
+	if save_archive == True:
+		timestamp = exptime
+		ar = pd.DataFrame({
+			'Path': pd.Series(contents),
+			'AllPagesText': b['AllPagesText'],
+			'Timestamp': timestamp
+			},index=range(0,pd.Series(contents).shape[0]))
+		arch = pd.concat([arch, ar],ignore_index=True)
+		arch.fillna('',inplace=True)
+		arch.dropna(inplace=True)
+		arch.to_pickle(path_out+".pkl.xz",compression="xz")
+
+
+	b.drop(columns=['AllPagesText','CaseInfoOutputs','ChargesOutputs','FeeOutputs','TotalD999','ChargesTable','FeeSheet'],inplace=True)
+	outputs = pd.concat([outputs, b],ignore_index=True)
+	
+	outputs.fillna('',inplace=True)
+	charges.fillna('',inplace=True)
+	fees.fillna('',inplace=True)
+
+	# write 
+	if out_ext == "xls":
+		with pd.ExcelWriter(path_out) as writer:
+			outputs.to_excel(writer, sheet_name="cases-table")
+			fees.to_excel(writer, sheet_name="fees-table")
+			charges.to_excel(writer, sheet_name="charges-table")
+	elif out_ext == "pkl":
+		outputs.to_pickle(path_out+".xz",compression="xz")
+	elif out_ext == "xz":
+		outputs.to_pickle(path_out,compression="xz")
+	elif out_ext == "json":
+		outputs.to_json(path_out)
+	elif out_ext == "csv":
+		outputs.to_csv(path_out,escapechar='\\')
+	elif out_ext == "md":
+		outputs.to_markdown(path_out)
+	elif out_ext == "txt":
+		outputs.to_string(path_out)
+	elif out_ext == "dta":
+		outputs.to_stata(path_out)
+	elif out_ext == "no_export" or print_log == True:
+		print(outputs.to_string())
+	else:
+		raise Exception("Output file extension not supported! Please output to .xls, .pkl, .json, or .csv")
 		
 	log_complete(conf, start_time)
 	on_batch = 0
@@ -363,7 +382,6 @@ def writeFees(conf):
 		feesheet = pd.concat(feesheet,axis=0,ignore_index=True) # add all dfs in batch -> df
 		fees = fees.append(feesheet, ignore_index=True) #pd.concat([fees, feesheet],axis=0,ignore_index=True)
 
-		console_log(conf, on_batch,exptime,'Parsing fee sheets to table...')
 
 		fees.fillna('',inplace=True)
 
@@ -428,7 +446,6 @@ def writeCharges(conf):
 		chargetabs = pd.concat(chargetabs,axis=0,ignore_index=True)
 		charges = charges.append(chargetabs,ignore_index=True)
 
-		console_log(conf, on_batch,exptime,'Exporting charges to table...')
 		
 		charges.fillna('',inplace=True)
 
@@ -455,7 +472,6 @@ def writeCharges(conf):
 		else:
 			raise Exception("Output file extension not supported! Please output to .xls, .json, or .csv")
 		on_batch += 1
-		console_log(conf, on_batch,exptime,'Exporting charges to table...')
 	log_complete(conf, start_time)
 	on_batch = 0
 
@@ -487,7 +503,6 @@ def search(conf, method, status=''):
 		if print_log == True:
 			print(allCustomOutputs)
 		on_batch += 1
-		console_log(conf, on_batch,exptime,status)
 	log_complete(conf, start_time)
 	return allCustomOutputs
 
@@ -542,7 +557,6 @@ def write(conf, method, status=''):
 		else:
 			raise Warning("Batch export failed!")
 		on_batch += 1
-		console_log(conf, on_batch,exptime,status)
 	log_complete(conf, start_time)
 	on_batch = 0
 
@@ -618,7 +632,7 @@ def getCaseInfo(text: str):
 		if bool(re.search(r'(?:DOB)(.{5,100})(?:Name)', text, re.MULTILINE)) == True:
 			name = re.search(r'(?:DOB)(.{5,100})(?:Name)', text, re.MULTILINE).group(1).replace(":","").replace("Case Number:","").strip()
 	try:
-		alias = re.search(r'(SSN)(.{5,75}+)(Alias)', text, re.MULTILINE).group(2).replace(":","").replace("Alias 1","").strip()
+		alias = re.search(r'(SSN)(.{5,75})(Alias)', text, re.MULTILINE).group(2).replace(":","").replace("Alias 1","").strip()
 	except (IndexError, AttributeError):
 		pass
 	else:
@@ -839,7 +853,7 @@ def log_complete(conf, start_time):
 /_/  |_/_/\\__,_/\\___/\\____/_/   \\__,_/\\___/_/     
 																																										
 	
-	ALACORDER beta 7.4.4
+	ALACORDER beta 7.4.8
 	by Sam Robson	
 
 	Searched {path_in} 
@@ -850,33 +864,3 @@ def log_complete(conf, start_time):
 
 ''') 
 
-def console_log(conf, on_batch: int, last_log, to_str):
-	path_in = conf['in_path']
-	path_out = conf['out_path'] if conf['out_path'] != "" else "----"
-	path_out = "Writing to " + conf['out_path'] if conf['out_path'] != "" else "No export path provided - did not export!"
-	case_max = conf['case_max']
-	bsize = conf['batchsize']
-	plog = conf['print_log']
-	tot_b = conf['tot_batches']
-	exptime = time.time()
-	if last_log == None:
-		last_log = exptime
-	expected_time = (exptime - last_log + 2) * (tot_b - on_batch) / 60
-	if plog == True:
-		print(f'''\n\n
-	    ___    __                          __         
-	   /   |  / /___  _________  _________/ /__  _____
-	  / /| | / / __ `/ ___/ __ \\/ ___/ __  / _ \\/ ___/
-	 / ___ |/ / /_/ / /__/ /_/ / /  / /_/ /  __/ /    
-	/_/  |_/_/\\__,_/\\___/\\____/_/   \\__,_/\\___/_/     
-																																											
-		
-		ALACORDER beta 7.4.4
-
-		Searching {path_in} 
-		{path_out} 
-
-		Processed {on_batch*bsize} of {case_max} total cases...
-		{to_str}
-
-	''') 
